package org.alltimeflashdreamer.log4jmonitor;

import java.io.EOFException;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;
import org.apache.log4j.Logger;
import org.apache.log4j.LogManager;
import org.apache.log4j.net.SocketAppender;
import org.apache.log4j.spi.LoggingEvent;

/**
 * Title: LoggingReceiver<br>
 * Description: Represents a daemon thread that processes connections from a
 * <code>org.apache.log4j.net.SocketAppender</code>. The Log4J messages created by
 * this class itself are sent to the (own) thread too.<br>
 * This code has been copied from the authors below.
 *
 * @author    <a href="mailto:chainsaw@puppycrawl.com">Oliver Burn</a>, revised by
 *            <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
 * @version   1.1
 */

class LoggingReceiver extends Thread
{
    /** where to put the events to * */
    private final MyTableModel model;
    /** server for listening for connections * */
    private final ServerSocket serverSocket;

    /** used to log messages * */
    private Logger logger;


    /**
     * Creates a new <code>LoggingReceiver</code> instance. The new instance
	 * appends its own Logger as a SocketAppender on the localhost, allowing
	 * it to log its own messages to the same output window.
     *
     * @param model                 model to place put received into
     * @param iPort                 port to listen on
     * @exception IOException       if an error occurs
	 * @see java.net.ServerSocket
	 * @see java.lang.Thread#setDaemon
	 * @see org.apache.log4j.Logger
	 * @see org.apache.log4j.net.SocketAppender
     */
    LoggingReceiver( MyTableModel model, int iPort )
        throws IOException
    {
        setDaemon( true );
        this.model = model;
        serverSocket = new ServerSocket( iPort );

		//build own Logger and append start logging to the server too
        logger = Logger.getLogger( this.getClass() );
        logger.addAppender( new SocketAppender( "localhost", iPort ) );
    }

    /**
	 * Listens for client connections. If a new one is accepted, a new Thread
	 * is started which listens to this client.
	 * @see java.net.ServerSocket#accept
	 * @see java.lang.Thread
	 */
    public void run()
    {
        logger.info( "LoggingReceiver thread started." );
        try
        {
            for ( ; ;  )
            {
                logger.debug( "Waiting for a connection" );
                final Socket client = serverSocket.accept();
                logger.info( "Accepted a connection from " +
                        client.getInetAddress().getHostName() + "." );
                final Thread t = new Thread( new Slurper( client ) );
                t.setDaemon( true );
                t.start();
            }
        } catch ( IOException e )
        {
            logger.error( "Caught IOException in accepting connections, stopping...", e );
        }
    }

    /**
     * Override of Object's finalize method to shut down the Log4J Logger.
     *
     * @exception Throwable  as defined in the superclass
	 * @see java.lang.Object#finalize
     */
    protected void finalize()
        throws Throwable
    {
        LogManager.shutdown();
    }

//-------------------------- inner class Slurper -----------------------------//
    /**
     * Helper that actually processes a client connection. It receives events and
     * adds them to the supplied model.
     *
	 * @author    <a href="mailto:chainsaw@puppycrawl.com">Oliver Burn</a>, revised by
	 *            <a href="mailto:florian@steinsiepe.ch">Florian Steinsiepe</a>
	 * @version   1.1
     */
    private class Slurper implements Runnable
    {
        /** socket connection to read events from */
        private final Socket client;

        /**
         * Creates a new <code>Slurper</code> instance.
         *
         * @param client  socket to receive events from
         */
        Slurper( Socket client )
        {
            this.client = client;
        }

        /**
		 * Loops the received events, adding each to the table model's list
		 * of shown events.
		 */
        public void run()
        {
            logger.debug( "Slurper: Starting to get data..." );
            try
            {
				//read the whole stream and (re)construct the LoggingEvents
                final ObjectInputStream ois = new ObjectInputStream( client.getInputStream() );
                while ( true )
                    model.addEvent( ( LoggingEvent )ois.readObject() );
            } catch ( EOFException e )
            {
                logger.info( "Slurper: Reached EOF, therefore closing connection." );
            } catch ( SocketException e )
            {
                logger.info( "Slurper: Caught SocketException, closing connection..." );
            } catch ( IOException e )
            {
                logger.warn( "Slurper: Caught IOException, closing connection...", e );
            } catch ( ClassNotFoundException e )
            {
                logger.warn( "Slurper: Caught ClassNotFoundException, closing connection...", e );
            }

            try
            {
                client.close();
                logger.debug( "Slurper: Successfully closed the connection to the client." );
            } catch ( IOException e )
            {
                logger.warn( "Slurper: Error closing connection", e );
            }
        }
    }
}
